/*
 **************************************************************************
 *
 * Boot-ROM-Code to load an operating system across a TCP/IP network.
 *
 * Module:  boot.c
 * Purpose: Perform the actual booting process
 * Entries: do_boot
 *
 **************************************************************************
 *
 * Copyright (C) 1995,1996 Gero Kuhlmann <gero@gkminix.han.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "../../headers/general.h"
#include "../public/net.h"
#include "../public/boot.h"
#include "../public/romlib.h"
#include "./bootpriv.h"
#include "./bootp.h"
#include "./load.h"


/*
 **************************************************************************
 *
 * Miscellaneous defines:
 */
#define INPUT_TIMEOUT	28	/* wait this many ticks for user input	*/



/*
 **************************************************************************
 * 
 * Print a string with a maximum number of characters. We have to use
 * printf so that all special characters get displayed correctly.
 */
void printstr(unsigned char *str, int strlen)
{
  while (*str && strlen > 0) {
	printf("%c", *(str++));
	strlen--;
  }
}



/*
 **************************************************************************
 * 
 * Print an IP number.
 */
static void printip(t_ipaddr ip)
{
  unsigned char *cp = &(unsigned char)ip;

  printf("%u.%u.%u.%u", cp[0], cp[1], cp[2], cp[3]);
}


/*
 **************************************************************************
 * 
 * Actually perform the booting process by first calling the BOOTP client,
 * and then loading the operating system using TFTP.
 *
 */
void do_boot(void)
{
  struct bootp *bp;

  /* Get BOOTP record from server */
#ifdef USEMENU
  printf("Press ESC for menu\n");
#endif
  if ((bp = bootp()) != NULL) {
	printf("\n\nLocal IP:  "); printip(ntohl(bp->bp_yiaddr));
	printf("\nServer IP: "); printip(ntohl(bp->bp_siaddr));
	printf(" ("); printstr(bp->bp_sname, sizeof(bp->bp_sname));
	printf(")\nFile name: "); printstr(bp->bp_file, sizeof(bp->bp_file));
	printf("\n\n\n");
  }
#ifdef USEMENU
  if (bp == NULL || getkey(INPUT_TIMEOUT) == CHR_ESC)
	if ((bp = domenu(bp)) == NULL)
		return;
#else
  if (bp == NULL)
	fatal();
  else if (getkey(INPUT_TIMEOUT) == CHR_ESC)
	return;
#endif

  /* Get the boot file from the server */
  if (!load(ntohl(bp->bp_siaddr), (char *)(bp->bp_file), sizeof(bp->bp_file)))
	goto dofatal;

  /* Finally call the new boot image */
  call_image(bp);

dofatal:
  fatal();
}

