/*
 **************************************************************************
 *
 * Boot-ROM-Code to load an operating system across a TCP/IP network.
 *
 * Module:  bootp.h
 * Purpose: Definitions for implementing the BOOTP protocol
 * Entries: none
 *
 **************************************************************************
 *
 * Copyright (C) 1995,1996 Gero Kuhlmann <gero@gkminix.han.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


/*
 **************************************************************************
 *
 * BOOTP hardware identifiers. Same as those used for ARP protocol. See
 * assigned numbers RFC.
 */
#define BOOTP_ETHER	1		/* Ethernet 10Mbps		*/
#define	BOOTP_EETHER	2		/* Experimental Ethernet	*/
#define	BOOTP_AX25	3		/* AX.25 Level 2		*/



/*
 **************************************************************************
 *
 * BOOTP protocol opcodes:
 */
#define	BOOTP_REQUEST	1		/* BOOTP request		*/
#define	BOOTP_REPLY	2		/* BOOTP reply			*/



/*
 **************************************************************************
 *
 * BOOTP UDP port numbers:
 */
#define BOOTP_S_PORT	67		/* BOOTP server port		*/
#define BOOTP_C_PORT	68		/* BOOTP client port		*/



/*
 **************************************************************************
 *
 * Various definitions:
 */
#define BOOTP_RETRY	32		/* Maximum number of retries	*/
#define MAX_EXT_BLOCKS	4		/* Max # of blocks in ext file	*/



/*
 **************************************************************************
 *
 * Vendor magic ID's:
 */
#define VM_SIZE		4			/* Size of vendor magic	*/
#define	VM_STANFORD	"STAN"			/* Stanford		*/
#define VM_RFC1048      "\143\202\123\143"	/* RFC 1048		*/



/*
 **************************************************************************
 *
 * Tag values for RFC1048 vendor fields:
 */
#define VEND_NOP	0		/* NOP				*/
#define VEND_SUBNET	1		/* Subnet mask			*/
#define VEND_TOFFS	2		/* Time offset			*/
#define VEND_ROUTER	3		/* Router IP address list	*/
#define VEND_TIMES	4		/* Time server IP address	*/
#define VEND_NAMES	5		/* Name server IP address	*/
#define VEND_DOMAINS	6		/* Domain name server IP addr	*/
#define VEND_LOGS	7		/* Log server IP address	*/
#define VEND_COOKIES	8		/* Cookie server IP address	*/
#define VEND_LPRS	9		/* LPR server IP address	*/
#define VEND_IMPRESS	10		/* IMPRESS server IP address	*/
#define VEND_RESOURCES	11		/* Resource location server IP	*/
#define VEND_HNAME	12		/* Client host name		*/
#define VEND_BFSIZE	13		/* Boot file size		*/
#define VEND_DUMPFILE	14		/* Merrit Dump file name	*/
#define VEND_DOMAIN	15		/* Client domain name		*/
#define VEND_SWAPS	16		/* Swap server IP address	*/
#define VEND_ROOTPATH	17		/* Root file system path name	*/
#define VEND_EXTFILE	18		/* Extensions file name		*/
#define VEND_END	255		/* End of option list		*/



/*
 **************************************************************************
 *
 * BOOTP packet:
 */
struct bootp
{
	unsigned char  bp_op;		/* BOOTP op code		*/
	unsigned char  bp_hwtype;	/* hardware type		*/
	unsigned char  bp_hlen;		/* length of hardware address	*/
	unsigned char  bp_hops;		/* hop-number, only used by gw	*/
	unsigned long  bp_xid;		/* transaction ID		*/
	unsigned short bp_secs;		/* time since client booted	*/
	unsigned short bp_unused;	/* unused			*/
	t_ipaddr       bp_ciaddr;	/* client IP address		*/
	t_ipaddr       bp_yiaddr;	/* your (client) IP address	*/
	t_ipaddr       bp_siaddr;	/* server IP address		*/
	t_ipaddr       bp_giaddr;	/* BOOTP gateway IP address	*/
	unsigned char  bp_chaddr[16];	/* client hardware address	*/
	unsigned char  bp_sname[64];	/* server host name		*/
	unsigned char  bp_file[128];	/* boot file name		*/
	unsigned char  bp_vend[64];	/* optional vendor information	*/
};



/*
 **************************************************************************
 *
 * Functions in BOOTP module:
 */

/* Get a BOOTP record from server */
extern struct bootp *bootp(void);

