;**************************************************************************
;*
;* Boot-ROM-Code to load an operating system across a TCP/IP network.
;*
;* Module:  basicio.asm
;* Purpose: Basic I/O functions for DOS simulator
;* Entries: dos01, dos02, dos03, dos04, dos05, dos06, dos07, dos08, dos09,
;*          dos0A, dos0B, dos0C, prnchr
;*
;**************************************************************************
;*
;* Copyright (C) 1995,1996 Gero Kuhlmann <gero@gkminix.han.de>
;*
;*  This program is free software; you can redistribute it and/or modify
;*  it under the terms of the GNU General Public License as published by
;*  the Free Software Foundation; either version 2 of the License, or
;*  any later version.
;*
;*  This program is distributed in the hope that it will be useful,
;*  but WITHOUT ANY WARRANTY; without even the implied warranty of
;*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;*  GNU General Public License for more details.
;*
;*  You should have received a copy of the GNU General Public License
;*  along with this program; if not, write to the Free Software
;*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;*


;
;**************************************************************************
;
; Include assembler macros:
;
include ..\..\headers\asm\macros.inc
include ..\..\headers\asm\layout.inc
include ..\..\headers\asm\memory.inc
include ..\public\minidos.inc
include .\dospriv.inc


;
;**************************************************************************
;
; Miscalleneous equates
;
AUX_INIT	equ	10100011b	; AUX port initialization value:
					;  2400 Baud, no parity, 1 stop bit,
					;  8 data bits
CHR_BS		equ	08h		; Backspace character
CHR_RET		equ	0Dh		; Return character
CHR_LF		equ	0Ah		; Line-feed character
CHR_FF		equ	0Ch		; Form-feed character
CHR_TAB		equ	09h		; Tab character


;
;**************************************************************************
;
; Data segment
;
data_start

		extrn	old_ds:word	; caller's DS register

ifdef NEEDEXTIO
auxflag		db	0		; flag if serial port initialized
prnflag		db	0		; flag if printer port initialized
endif

data_end


;
;**************************************************************************
;
; Start code segment.
;
text_start

ifdef NEEDLINEIO
ifdef NEEDFILEIO
	extrn	dos3F:near
endif
endif

	public	dos01, dos02, dos03	; define entry points
	public	dos04, dos05, dos06
	public	dos07, dos08, dos09
	public	dos0A, dos0B, dos0C

	public	prnchr


;
;**************************************************************************
;
; Read character from keyboard and then send it to the screen
; Input:  none
; Output: AL  -  character
; Registers changed: AL
;
dos08		label	near
dos07		label	near
dos01		proc	near

	push	bx
	mov	bh,ah			; save AH
	call	keywait			; wait for character from keyboard
	mov	bl,al
	cmp	bh,1			; only print character with fn 01h
	jne	short dos011
	call	prnchr			; print character
dos011:	mov	ax,bx			; restore AH
	pop	bx
	ret

dos01		endp


;
;**************************************************************************
;
; Print character onto screen
; Input:  DL  -  character
; Output: none
; Registers changed: none
;
dos02		proc	near

	push	ax
	mov	al,dl
	call	prnchr			; print character
	pop	ax
	ret

dos02		endp


;
;**************************************************************************
;
; Receive character on serial port COM1
; Input:  none
; Output: AL  -  character
; Registers changed: AL
;
dos03		proc	near

ifdef NEEDEXTIO
	push	bx
	mov	bh,ah
	mov	al,DEV_AUX
	call	devinit			; initialize serial device
	mov	ah,02h
	int	14h			; receive character
	mov	ah,bh			; restore AH
	pop	bx
endif

	ret

dos03		endp


;
;**************************************************************************
;
; Send character to serial port COM1
; Input:  DL  -  character
; Output: none
; Registers changed: none
;
dos04		proc	near

ifdef NEEDEXTIO
	push	ax
	push	dx
	mov	al,DEV_AUX
	call	devinit			; initialize serial device
	pop	ax
	push	ax			; restore character
	mov	ah,01h
	int	14h			; send character
	pop	dx
	pop	ax
endif

	ret

dos04		endp


;
;**************************************************************************
;
; Send character to printer port
; Input:  DL  -  character
; Output: none
; Registers changed: none
;
dos05		proc	near

ifdef NEEDEXTIO
	push	ax
	push	dx
	mov	al,DEV_PRN
	call	devinit			; initialize parallel port
	pop	ax
	push	ax			; restore character
	xor	ah,ah
	int	17h			; send character
	pop	dx
	pop	ax
endif

	ret

dos05		endp


;
;**************************************************************************
;
; Direct console I/O
; Input:  DL  -  character for console output, or 0FFh for console input
; Output: AL  -  character from console input, or 0 for console output
; Changed registers: AL
;
dos06		proc	near

	xor	al,al
	cmp	dl,0FFh			; check for function
	jne	short dos02		; print character

	mov	ah,01h
	int	16h			; get character from keyboard
	jz	short dos062		; got no character
	xor	ah,ah
	int	16h			; get character from buffer
	jmp	short dos063
dos062:	xor	al,al			; set return value 0
dos063:	mov	ah,06h			; restore AH
	or	al,al			; set zero flag
	ret

dos06		endp


;
;**************************************************************************
;
; Print character string onto screen. String is terminated by '$' character.
; Input:  DS:DX  -  pointer to string
; Output: none
; Registers changed: none
;
dos09		proc	near

	cld
	push	ax
	push	si
	push	ds
	mov	ds,old_ds		; get caller's DS
	mov	si,dx
dos091:	lodsb				; get next character
	cmp	al,'$'			; end of string?
	je	short dos092
	call	prnchr			; print character
	jmp	short dos091		; proceed with next character

dos092:	pop	ds
	pop	si
	pop	ax
	ret

dos09		endp


;
;**************************************************************************
;
; Read string from keyboard. The first character of the buffer contains
; the buffer size, the next character will receive the actual number
; of characters read. Note that with file I/O enabled we just need to
; use DOS function 3F.
; Input:  DS:DX  -  pointer to buffer
; Output: none
; Registers changed: none
;
dos0A		proc	near

ifndef NEEDLINEIO

	push	es
	push	bx
	mov	bx,old_ds
	mov	es,bx
	mov	bx,dx			; simply mark the buffer as
	mov	byte ptr es:[bx+1],0	; empty
	pop	bx
	pop	es
	ret

else					;; NEEDLINEIO

ifndef NEEDFILEIO

	push	ax
	push	bx
	push	di
	push	es
	mov	es,old_ds
	mov	di,dx
	xor	bx,bx			; BX - offset into buffer

dos0A1:
	cmp	bl,es:[di]		; end of buffer reached?
	jae	short dos0A4
	call	keywait			; get next character

	cmp	al,CHR_BS		; backspace?
	jne	short dos0A2
	or	bx,bx			; don't delete past beginning of
	jz	short dos0A1		; buffer
	dec	bx			; kill character in the buffer and
	jmp	short dos0A5		; on the screen

dos0A2:	cmp	al,CHR_RET		; carriage return?
	je	short dos0A3
	mov	es:[di+bx+2],al		; save character
	inc	bx
dos0A5:	call	prnchr			; and print it
	jmp	short dos0A1

dos0A3:	call	prnchr
	mov	al,CHR_LF		; print CR/LF
	call	prnchr
dos0A4:	mov	es:[di+1],bl		; save length in the buffer
	pop	es
	pop	di
	pop	bx
	pop	ax
	ret

else					;; NEEDFILEIO

	push	ax
	push	bx
	push	cx
	push	es
	mov	es,old_ds		; ES:BX points to buffer
	mov	bx,dx
ifdef IS386
	movzx	cx,byte ptr es:[bx]	; get maximum length
else
	mov	cl,byte ptr es:[bx]	; get maximum length
	xor	ch,ch
endif
	inc	dx			; let the pointer show to the buffer
	inc	dx
	mov	bx,RES_STDIN		; we always use the console
	call	dos3F
	jc	short dos0A9		; don't do enything in case of error
	dec	dx			; restore original buffer pointer
	mov	bx,dx
	mov	byte ptr es:[bx],al	; save buffer length
	dec	dx
dos0A9:	pop	es
	pop	cx
	pop	bx
	pop	ax
	ret

endif					;; NEEDFILEIO
endif					;; NEEDLINEIO

dos0A		endp


;
;**************************************************************************
;
; Get keyboard status
; Input:  none
; Output: AL  -  0FFh -> key pressed, 000h -> no key pressed
; Changed registers: AL
;
dos0B		proc	near

dos0B1:	xor	al,al			; AL remains zero if no character in
	mov	ah,01h			; keyboard buffer
	int	16h			; check BIOS keyboard buffer
	jz	short dos0B9		; no character in buffer

	or	al,al			; check for extended character
	jnz	short dos0B2
	xor	ah,ah
	int	16h			; discard extended character
	jmp	short dos0B1

dos0B2:	mov	al,0FFh			; yes, we have a character available
dos0B9:	mov	ah,0Bh			; restore AH
	ret

dos0B		endp


;
;**************************************************************************
;
; Clear input buffer and then call input routine.
; Input:  AL  -  input function number
; Output: AL  -  input character
; Registers changed: depending on input function
;
dos0C		proc	near

	push	ax
dos0C1:	mov	ah,01h
	int	16h			; get keyboard status
	jz	short dos0C2		; no more characters in buffer
	xor	ah,ah
	int	16h			; get character from input buffer
	jmp	short dos0C1		; repeat until buffer empty
dos0C2:	pop	ax

dos0C4:	cmp	al,06h
ifdef IS386
	je	dos06			; call DOS function 06
else
	jne	short dos0C5
	jmp	dos06			; call DOS function 06
endif

dos0C5:	cmp	al,0Ah
ifdef IS386
	je	dos0A			; call DOS function 0A
else
	jne	short dos0C6
	jmp	dos0A			; call DOS function 0A
endif

dos0C6:	mov	ah,al
	jmp	dos01			; call DOS function 01

dos0C		endp


;
;**************************************************************************
;
; Wait for a key press
; Input:  none
; Output: AL  -  character
; Registers changed: AX
;
keywait		proc	near

keyw1:	xor	ah,ah
	int	16h			; get character from keyboard
	or	al,al			; check for extended keyboard character
	jz	short keyw1
keyw9:	ret

keywait		endp


;
;**************************************************************************
;
; Print character onto console
; Input:  AL  -  character to print
; Output: none
; Registers changed: AX
;
prnchr		proc	near

	push	bx
	push	cx
	push	dx
	cmp	al,CHR_FF		; skip form feed character
	je	short prnch9
	cmp	al,CHR_TAB		; tabulator is special
	je	short prnch2
	cmp	al,CHR_LF		; line feed is special as well
	je	short prnch4
	mov	bx,0007h		; set color and screen page
	mov	ah,0Eh
	int	10h			; print character
	jmp	short prnch9

; Handle tabulator

prnch2:	mov	ah,03h			; read cursor position
	xor	bh,bh
	int	10h
	and	dl,0F8h			; set new cursor position
	add	dl,008h
prnch5:	mov	ah,02h			; set new cursor position
	xor	bh,bh
	int	10h
	jmp	short prnch9

; Handle line feed

prnch4:	mov	ah,03h			; read cursor position
	xor	bh,bh
	int	10h
	inc	dh
	cmp	dh,25			; at bottom of screen?
	jb	short prnch5		; no, just set new position
	mov	ax,0601h		; scroll screen up one line
	mov	bx,0700h		; set color
	xor	cx,cx			; upper left corner
	mov	dx,0FFFFh		; lower right corner
	int	10h

prnch9:	pop	dx
	pop	cx
	pop	bx
	ret

prnchr		endp


ifdef NEEDEXTIO
;
;**************************************************************************
;
; Initialize a device.
; Input:  AL  -  device type (CON, AUX, PRN)
; Output: none
; Registers changed: AX
;
devinit		proc	near

	push	dx
	mov	ah,1			; default device flag
	xor	dx,dx			; default device number

; Initialize serial port

	cmp	al,DEV_AUX
	jne	short devin2
	xchg	ah,auxflag		; already initialized?
	or	ah,ah
	jnz	short devin9
	mov	ax,AUX_INIT
	int	14h			; initialize serial port
	jmp	short devin9

; Initialize parallel port

devin2:	cmp	al,DEV_PRN
	jne	short devin9
	xchg	ah,prnflag		; already initialized?
	or	ah,ah
	jnz	short devin9
	mov	ah,01h
	int	17h			; initialize printer port

devin9:	pop	dx
	ret

devinit		endp
endif					;; NEEDEXTIO


;
;**************************************************************************
;
text_end

	end

