/*
 **************************************************************************
 *
 * Boot-ROM-Code to load an operating system across a TCP/IP network.
 *
 * Module:  icmp.h
 * Purpose: Definitions for implementing the internet control message protocol
 * Entries: none
 *
 **************************************************************************
 *
 * Copyright (C) 1995,1996 Gero Kuhlmann <gero@gkminix.han.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


/*
 **************************************************************************
 *
 * ICMP messages:
 */
#define ICMP_ECHO_REP	0	/* Echo reply				*/
#define ICMP_DEST_UNRCH	3	/* Destination unreachable		*/
#define ICMP_SRC_QUENCH	4	/* Source Quench request		*/
#define ICMP_REDIRECT	5	/* IP packet redirect info		*/
#define ICMP_ECHO_REQ	8	/* Echo request				*/
#define ICMP_TTL_EXCEED	11	/* Packet time exceeded			*/
#define ICMP_PARAM_PROB	12	/* Parameter problem			*/



/*
 **************************************************************************
 *
 * Codes for ICMP_DEST_UNRCH
 */
#define ICMP_NET_UNRCH	0	/* Network unreachable			*/
#define ICMP_HOST_UNRCH	1	/* Host unreachable			*/
#define ICMP_PROT_UNRCH	2	/* Protocol unreachable			*/
#define ICMP_PORT_UNRCH	3	/* Port unreachable			*/
#define ICMP_NEED_FRAG	4	/* Fragmentation needed			*/
#define ICMP_SRC_FAIL	5	/* Source route failed			*/



/*
 **************************************************************************
 *
 * Codes for ICMP_TTL_EXCEED
 */
#define ICMP_EXC_TTL	0	/* TTL count exceeded			*/
#define ICMP_EXC_FRAG	1	/* Fragment Reass time exceeded		*/



/*
 **************************************************************************
 *
 * Codes for ICMP_REDIRECT
 */
#define ICMP_RD_NET	0	/* Redirect net				*/
#define ICMP_RD_HOST	1	/* Redirect host			*/
#define ICMP_RD_NETTOS	2	/* Redirect net for TOS			*/
#define ICMP_RD_HOSTTOS	3	/* Redirect host for TOS		*/



/*
 **************************************************************************
 *
 * Miscellaneous defines
 */
#define ICMP_DATA_SIZE	64	/* number of bytes from orig datagram	*/



/*
 **************************************************************************
 *
 * general ICMP message structure
 */
struct icmphdr {
	unsigned char  icmp_type;
	unsigned char  icmp_code;
	unsigned short icmp_check;
	unsigned long  icmp_val1;
	unsigned long  icmp_val2;
};

