;**************************************************************************
;*
;* Boot-ROM-Code to load an operating system across a TCP/IP network.
;*
;* Module:  packet.asm
;* Purpose: Interface to the packet driver
;* Entries: _reg_type, _write_packet, _get_mtu, _get_hw_addr, _pktpoll
;*
;**************************************************************************
;*
;* Copyright (C) 1995,1996 Gero Kuhlmann <gero@gkminix.han.de>
;*
;*  This program is free software; you can redistribute it and/or modify
;*  it under the terms of the GNU General Public License as published by
;*  the Free Software Foundation; either version 2 of the License, or
;*  any later version.
;*
;*  This program is distributed in the hope that it will be useful,
;*  but WITHOUT ANY WARRANTY; without even the implied warranty of
;*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;*  GNU General Public License for more details.
;*
;*  You should have received a copy of the GNU General Public License
;*  along with this program; if not, write to the Free Software
;*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;*


;
;**************************************************************************
;
; Include assembler macros:
;
include ..\..\headers\asm\macros.inc
include ..\..\headers\asm\layout.inc
include ..\..\headers\asm\memory.inc
include .\pktpriv.inc


;
;**************************************************************************
;
; Define entry for packet type table:
;
type_entry	struc

type_id		dw	?		; ID for packet type
type_driver	dw	?		; pointer to type event routine
type_handle	dw	?		; packet driver handle for type

type_entry	ends


;
;**************************************************************************
;
; BSS segment:
;
bss_start

		public	bufsize, readbuf, writebuf

readtype	dw	?		; type of packet in buffer
readsize	dw	?		; actual size of data in buffer
readbuf		dw	?		; pointer to first read buffer
writebuf	dw	?		; pointer to write buffer
bufsize		dw	?		; size of read and write buffers
hwaddr		db	ETH_ALEN dup (?); hardware address

typetab		type_entry	TYPENUM dup (<>)

bss_end


;
;**************************************************************************
;
; Start code segment.
;
text_start

	public	_reg_type		; define entry points
	public	_write_packet
	public	_get_mtu
	public	_get_hw_addr
	public	_pktpoll


;
;**************************************************************************
;
; Register a new packet type.
; Input:  1. arg  -  type identifier
;         2. arg  -  near pointer to poll callback routine
; Output: error code
;
_reg_type	proc	near

	penter				; setup standard stack frame
	push	es
	push	di
	push	si
	mov	si,offset dgroup:typetab
regt1:	mov	ax,type_id[si]
	or	ax,ax
	jz	short regt3		; is the slot available?
	add	si,size type_entry
	jmp	short regt1

regt3:	getarg	ax,0
	mov	type_id[si],ax		; save type id
	getarg	ax,1			; get pointer to handler routine
	mov	type_driver[si],ax
	push	si
	mov	di,offset cgroup:receiver
	mov	ax,cs
	mov	es,ax			; ES:DI  -  pointer to receive handler
	lea	si,type_id[si]		; DS:SI  -  pointer to type ID
	mov	cx,size type_id		; CX     -  size of type ID
	mov	al,CL_DEFAULT		; AL     -  default class
	mov	bx,TYPE_DEFAULT		; BX     -  default type
	xor	dl,dl			; DL     -  default interface number
	mov	ah,ACCESS_TYPE
	int	PKTINT			; setup access type
	pop	si
	jnc	short regt4
regt5:
ifdef IS386
	movzx	ax,dh			; prepare return value
else
	mov	al,dh			; in case of error prepare return
	xor	ah,ah			; value, e.g. error code
endif
	neg	ax
	jmp	short regt9

regt4:	mov	type_handle[si],ax	; save access type handle
	cmp	si,offset dgroup:typetab; check if hardware address already
	jnz	short regt7		; initialized
	mov	bx,ax
	mov	ax,ds
	mov	es,ax
	mov	di,offset dgroup:hwaddr
	mov	cx,length hwaddr	; get hardware address for current
	mov	ah,GET_ADDRESS		; handle from packet driver
	int	PKTINT
	jc	short regt5
regt7:	xor	ax,ax			; return without error
regt9:	pop	si
	pop	di
	pop	es
	pleave
	ret

_reg_type	endp


;
;**************************************************************************
;
; Send a packet using the packet driver
; Input:  1. arg  -  near pointer to data buffer
;         2. arg  -  number of bytes in buffer
;         3. arg  -  packet type value
;         4. arg  -  near pointer to destination hardware address
; Output: error code
;
_write_packet	proc	near

	penter				; setup standard stack frame
	cld
	push	es
	push	si
	push	di
	mov	ax,ds
	mov	es,ax
	getarg	ax,2			; get packet type
	mov	bx,offset dgroup:typetab
	mov	cx,length typetab
wrtp1:	cmp	ax,type_id[bx]		; find packet type in type table
	je	short wrtp2
	add	bx,size type_entry
	loop	short wrtp1
	mov	ax,-ERR_BAD_TYPE	; type not found
	jmp	short wrtp9

wrtp2:	mov	di,writebuf
	mov	cx,ETH_ALEN		; copy destination address into
	getarg	si,3			; send buffer
	rep	movsb
	mov	cx,ETH_ALEN		; copy source address into send
	mov	si,offset dgroup:hwaddr	; buffer
	rep	movsb
	stosw				; copy packet type
	getarg	cx,1			; get number of characters to write
	getarg	si,0			; get data pointer
	cmp	cx,ETH_DATA_MAX
	jbe	short wrtp4		; check if maximum number of data
	mov	cx,ETH_DATA_MAX		; bytes exceeded
wrtp4:	mov	ax,cx
	rep	movsb			; copy payload into output buffer
	cmp	ax,ETH_DATA_MIN
	jae	short wrtp5		; check if minimum number of data
	mov	cx,ETH_DATA_MIN		; bytes copied
	sub	cx,ax
	xor	al,al			; pad with zeroes if not
	rep	stosb
	mov	ax,ETH_DATA_MIN

wrtp5:	mov	si,writebuf		; set pointer to write buffer
	mov	bx,type_handle[bx]	; set packet driver handle
	mov	cx,ax
	add	cx,ETH_HLEN		; set packet buffer size
	mov	ah,SEND_PKT		; now send the packet
	int	PKTINT
	jnc	short wrtp8
ifdef IS386
	movzx	ax,dh			; return with error
else
	mov	al,dh
	xor	ah,ah			; return with error
endif
	neg	ax
	jmp	short wrtp9

wrtp8:	xor	ax,ax			; no error
wrtp9:	pop	di
	pop	si
	pop	es
	pleave
	ret

_write_packet	endp


;
;**************************************************************************
;
; Return Maximum Transfer Unit.
; Input:  none
; Output: MTU
;
_get_mtu	proc	near

	mov	ax,bufsize
	sub	ax,ETH_HLEN		; MTU is buffer size minus header
	ret				; length

_get_mtu	endp


;
;**************************************************************************
;
; Return hardware address.
; Input:  1. arg  -  near pointer to buffer
; Output: none
;
_get_hw_addr	proc	near

	penter				; setup standard stack frame
	cld
	push	es
	push	si
	push	di
	mov	ax,ds
	mov	es,ax
	getarg	di,0
	mov	si,offset dgroup:hwaddr
	mov	cx,ETH_ALEN		; just copy the hardware address
	rep	movsb
	pop	di
	pop	si
	pop	es
	pleave
	ret

_get_hw_addr	endp


;
;**************************************************************************
;
; This routine gets called by the packet driver when a new packet arrives.
; Input:  AX  -  function number: 0 = return buffer pointer, 1 = buffer filled
;         BX  -  handle
;         CX  -  length of data received (only when AX = 1)
;         DS:SI  -  pointer to receive buffer (only when AX = 1)
; Output: CX     -  length of buffer (only when AX = 0)
;         ES:DI  -  pointer to receive buffer (only when AX = 0)
; Registers changed: AX, BX, CX, DX, ES, SI, DI
;
receiver	proc	far

	push	ds
	mov	dx,NEWDATA
	mov	ds,dx
	or	ax,ax			; check function code
	jnz	short rec3

; Return address of receive buffer in ES:DI. Discard the packet if the
; receive buffer is not empty.

	mov	es,ax
	mov	di,ax			; discard packet if no free buffer
	mov	ax,readsize		; buffer free?
	or	ax,ax
	jnz	short rec9
	mov	es,dx
	mov	di,readbuf		; return pointer to read buffer
	jmp	short rec9

; The buffer has been read. Now mark it as busy, so that the polling routine
; can give it up one level.

rec3:	mov	si,offset dgroup:typetab
	mov	dx,length typetab
rec4:	cmp	bx,type_handle[si]	; find handle in type table
	je	short rec5
	add	si,size type_entry
	dec	dx
	jnz	short rec4		; discard packet if handle not found
	jmp	short rec9

rec5:	mov	dx,type_id[si]
	mov	readsize,cx		; save buffer size and mark buffer
	mov	readtype,dx		; busy
rec9:	pop	ds
	ret

receiver	endp


;
;**************************************************************************
;
; Get a received packet out of the read buffer, and call the upper layer
; handler, which gets the following parameters on it's stack:
;	char *buf    -  pointer to receive buffer
;	int bufsize  -  number of data bytes in receive buffer
;	char *addr   -  pointer to sender's address
;
; Input:  none
; Output: none
;
_pktpoll	proc	near

	mov	cx,readsize
	or	cx,cx			; immediately terminate if no
	jz	short pktp9		; packet received

	mov	bx,readbuf
	lea	ax,ETH_ALEN[bx]		; push pointer to source hardware
	push	ax			; address
	sub	cx,ETH_HLEN
	push	ax			; push buffer size (without header)
	add	bx,ETH_HLEN
	push	bx			; push buffer pointer (without header)

	mov	bx,offset dgroup:typetab
	mov	cx,length typetab
	mov	dx,readtype
pktp1:	cmp	dx,type_id[bx]
	je	short pktp2
	add	bx,size type_entry	; search type in type list
	loop	short pktp1
	jmp	short pktp3

pktp2:	mov	ax,1
	mov	bx,type_driver[bx]	; call type handler
	or	bx,bx
	jz	short pktp3
	call	bx
pktp3:	add	sp,6			; cleanup stack
	mov	readsize,0		; discard packet
pktp9:	ret

_pktpoll	endp


;
;**************************************************************************
;
text_end

	end

